import React, {Component, Fragment} from 'react';
import _ from 'lodash';
import {connect} from 'react-redux'
import {withRouter, Route, Switch, Redirect} from 'react-router-dom'
import {Query, withApollo} from 'react-apollo'
import ContentWrapper from './components/ContentWrapper'
import TabsHeader from './components/tabs/TabsHeader'
import {GET_CLUSTERS_QUERY} from './queries'
import Discover from './components/discover/Discover'
import WizardStarter from './components/tierClusterWizard/WizardStarter'
import WizardHeader from './components/wizard/WizardHeader'
import Welcome from './components/discover/Welcome'
import LoadingIcon from './components/widgets/LoadingIcon'
import MoreInfo, {InfoHeader} from "./components/tabs/MoreInfo";
import Isvg from "react-inlinesvg";
import {STATUS_SUCCESS} from "./actions";
import ClustersDashboard, {ClustersDashboardHeader} from "./components/tabs/ClustersDashboardNew";
import TabsFooter from './components/tabs/TabsFooter'
import Intercom from "./components/intercom/Intercom";
import Oops from './components/Oops';
import './app.scss';

const LoginRedirectToPath = () => {
    window.auth.loginRedirect({}, {
        service: "cloudtiering",
        startOnSignup: false
    }, {pathname: window.location.pathname});
    return <div></div>
};
const AuthenticatedRoute = props => props.auth.authenticated ? <Route {...props}/> : <LoginRedirectToPath/>;

const UnderConstruction = (title) => <div
    style={{width: '300px', height: '100%', margin: 'auto', position: 'relative'}}>{title} Under Construction...</div>;

const AppRoutes = ({auth, clusters, error}) => {
    return (
        <Switch>
            <AuthenticatedRoute auth={auth} path="/clusters" exact
                                render={() => <ContentWrapper content={ClustersDashboard}
                                                              componentHeader={TabsHeader}
                                                              secondaryHeader={ClustersDashboardHeader}
                                                              footer={TabsFooter}/>}/>
            <AuthenticatedRoute auth={auth} path="/license" exact
                                render={() => <ContentWrapper content={UnderConstruction.bind(this, "license")}
                                                              componentHeader={TabsHeader}/>}/>
            <AuthenticatedRoute auth={auth} path="/timeline" exact
                                render={() => <ContentWrapper content={UnderConstruction.bind(this, "timeline")}
                                                              componentHeader={TabsHeader}/>}/>
            <AuthenticatedRoute auth={auth} path="/discover" exact render={() => <Discover/>}/>
            <AuthenticatedRoute auth={auth} path="/welcome" exact render={() => <ContentWrapper content={Welcome} noNavigationToDashboard={true}/>}/>
            <AuthenticatedRoute auth={auth} path="/tier/:clusterId/volumes"
                                render={() => <ContentWrapper content={WizardStarter} nakedHeader={true}
                                                              secondaryHeader={WizardHeader} bodyClass="wizard"
                                                              data={{clusters, wizardType: "tier"}}/>}/>
            <AuthenticatedRoute auth={auth} path="/retier/:clusterId/volumes"
                                render={() => <ContentWrapper content={WizardStarter} nakedHeader={true}
                                                              secondaryHeader={WizardHeader} bodyClass="wizard"
                                                              data={{clusters, wizardType: "retier"}}/>}/>
            <AuthenticatedRoute auth={auth} path="/:clusterId/info"
                                render={() => <ContentWrapper content={MoreInfo}
                                                              secondaryHeader={InfoHeader}
                                                              data={{clusters}}/>}/>
            <AuthenticatedRoute auth={auth} path="/errorPage" exact render={() => <ContentWrapper content={Oops}
                                                                                                  nakedHeader={true}
                                                                                                  data={error}/>}/>
            <Redirect to={!error && clusters.length === 0 ? "/welcome" : "/clusters"}/>
        </Switch>
    );
};


class App extends Component {
    componentDidMount() {
        const {error, history} = this.props;
        if (error) history.replace('/errorPage');
    }

    render() {
        const {auth, clusters, error} = this.props;
        document.body.className = "with-sso"; //to add the widget
        return (
            <Fragment>
                <Intercom/>
                <AppRoutes auth={auth} clusters={clusters} error={error}/>
            </Fragment>
        );
    }
}


const WithRouterApp = withRouter(App);

const mapStateToProps = state => {
    return {
        auth: state.auth.authenticated ? state.auth : null,
        routing: state.router
    };
};


class InitialQuery extends Component {
    async componentDidMount() {
        console.log(`%c APP clearing store due to first time navigation`, 'font-weight:bold; color:purple');
        await this.props.client.clearStore();
    }

    render() {
        const {auth} = this.props;
        if (!auth || auth.status !== STATUS_SUCCESS) {
            return <div></div>;
        } else {
            return (
                <Query query={GET_CLUSTERS_QUERY}>
                    {({loading, error, data}) => {
                        if (loading)
                            return (<LoadingIcon text={"Cloud Tiering"} icon={<Isvg
                                src={require('./assets/svgs/logo.svg')}/>}/>);
                        const isAuthenticationError = (error) =>  (_.get(error, 'networkError.statusCode') === 401 || (_.get(error, 'graphQLErrors[0].extensions.code') === 401));
                        const errorMessage = error ? (isAuthenticationError(error) ? null : error.message) : null;
                        const clusters = !error && data.clusters ? data.clusters : null;
                        if (!clusters) console.log(`APP unable to get clusters error ${JSON.stringify(error)} data ${JSON.stringify(data)}`);
                        return (
                            <Fragment>
                                {clusters && <WithRouterApp auth={auth} clusters={clusters} error={errorMessage}/>}
                                {!clusters && ! errorMessage && <LoadingIcon text={"Clusters"} icon={<Isvg src={require('./assets/svgs/review-cluster.svg')}/>}/>}
                            </Fragment>
                        )
                    }}
                </Query>
            )
        }
    }
};

export default connect(mapStateToProps)(withApollo(InitialQuery));