import React, {Component} from 'react';
import {Query} from 'react-apollo'
import {GET_CLUSTERS_QUERY} from '../../queries'
import {routesObject} from "../../actions";
import Isvg from 'react-inlinesvg';
import ItemCountWidget from "../widgets/ItemCountWidget";
import SearchWidget from "../widgets/SearchWidget";
import {calculateSavings, defaultFilterWithKeys} from "./../../utils/utils"
import {connect} from "react-redux";
import LoadingIcon from '../widgets/LoadingIcon'
import classNames from 'classnames';
import {CLUSTER_STATUS, DASHBOARD_TITLES, TIER_BUTTON_TEXT} from "../../utils/consts";
import ReactPopover from "../reactPopover/ReactPopover";
import localStorage from "local-storage";
import routingInfoDispatcher from '../../dispatchers/routingInfoDispatcher'
import {CardItem, Estimated, ValuesContainer} from "./CardComponents";
import _ from "lodash";

require("./clustersDashboardNew.scss");

export const ClustersDashboardHeader = ({addCluster}) => (<div className="clusters-header">
    <h3>Cluster Dashboard</h3>
    <button type="button" className="add-cluster"
            onClick={() => {routesObject.routes.push('/discover')}}><Isvg
        cacheGetRequests={true}
        src={require("./../../assets/svgs/plus-fill-circle-icon.svg")}></Isvg>
    </button>
</div>);



const ClusterName = ({cluster, moreInfoClicked}) => {
    return (
        <div className="left-blue">
            <div className={classNames("name no-wrap", {italic: !cluster.name})} title={cluster.name ? cluster.name : ""}>{cluster.name ? cluster.name : "Unknown Cluster"}</div>
            {cluster.status.status === CLUSTER_STATUS.ON && <button type="button" onClick={()=> {
                moreInfoClicked();
                setTimeout(()=> {
                    routesObject.routes.push(`/${cluster.clusterId}/info`);
                }, 200)
            }}><div className="more-info-button">More Info<Isvg cacheGetRequests={true} src={require('../../assets/svgs/more-info-arrow.svg')}/></div></button>}
        </div>
    )
};

const ClusterStatus = ({cluster}) => {
    const {status: {status, message: errorMessage}} = cluster;
    return (
        <div className={classNames("status", {error: errorMessage || status === CLUSTER_STATUS.UNKNOWN})}>
            {status !== CLUSTER_STATUS.ON && <Isvg cacheGetRequests={true}
                                                   src={require(`./../../assets/svgs/cluster-status-red.svg`)}/>}
            {status === CLUSTER_STATUS.ON && <Isvg cacheGetRequests={true}
                                                   src={require(`./../../assets/svgs/cluster-status-green.svg`)}/>}
            {status === CLUSTER_STATUS.ON ? "Operational Cluster" : "Failed"}
            {status !== CLUSTER_STATUS.ON && <ReactPopover popoverClass="cluster-error-info" trigger="mouseover"
                                                           container={<button className="error-info" type="button"> <Isvg cacheGetRequests={true} src={require("./../../assets/svgs/info-circle-icon.svg")}/></button>}>
                {errorMessage || "Unknown Error"}
            </ReactPopover>}
        </div>
    )
};

const ClusterCard = ({cluster, handleTierCluster, moreInfoClicked}) => {
    const {volumes, capacityInformation} = cluster;
    const eligibleVolumes = _.get(volumes, "eligibleVolumes", 0);
    const tieredVolumes = _.get(volumes, "tieredVolumes", 0);
    // console.log(`%c Dashboard Cluster ${JSON.stringify(cluster)}`, 'font-weight:bold; color:purple');

    const estimated = _.get(capacityInformation, "performanceCapacity.isEstimatedInactiveCapacity", false);
    const {currentSavings, additionalSavings} = calculateSavings(cluster);

    return (
        <div className="cluster-card">
            <ClusterName cluster={cluster} moreInfoClicked={moreInfoClicked}/>
            <div className="items-container">
                <CardItem cluster={cluster} title="Tiering Health" icon={<Isvg cacheGetRequests={true}
                                                                                   src={require(`./../../assets/svgs/health.svg`)}/>}>
                    <ClusterStatus cluster={cluster}/>
                </CardItem>
                <CardItem cluster={cluster} title="Current Savings" icon={<Isvg cacheGetRequests={true}
                                                                                   src={require(`./../../assets/svgs/current-savings.svg`)}/>}>
                    <ValuesContainer values={currentSavings}/>
                </CardItem>
                <CardItem cluster={cluster} title={ `${tieredVolumes > 0 ? "Additional" :""} Saving Opportunities`}
                          className={estimated ? "estimated" : ""}
                          icon={<Isvg cacheGetRequests={true} src={require(`./../../assets/svgs/additional-savings.svg`)}/>}>
                    {estimated && <Estimated text={"Estimated Amount"} popoverText={DASHBOARD_TITLES.estimated}/>}
                    <ValuesContainer values={additionalSavings}/>
                    {eligibleVolumes > tieredVolumes && <button type="button" className="tier-button" onClick={handleTierCluster}>{TIER_BUTTON_TEXT}</button>}
                </CardItem>
            </div>
        </div>
    )
};

class ClustersDashboard extends Component {
    constructor(props, context) {
        super(props, context);
        this.state = {filter: null, moreInfoClicked: false};
    }

    moreInfoClicked() {
        this.setState({moreInfoClicked: true});
    }

    componentDidMount() {
        //clear all selections of last wizard
        localStorage.remove("selectedVolumes");
        localStorage.remove("selectedAwsKeys");
        localStorage.remove("clusterInfo");
        this.props.clearUrlParams();
    }

    setFilter(filter) {
        this.setState({filter});
    }

    tierCluster(cluster) {
        const {clusterId, tierInformation} = cluster;
        const nextRoute = _.get(tierInformation, "bucketName", null) ? `/retier/${clusterId}/volumes` : `/tier/${clusterId}/volumes`;
        routesObject.routes.push(nextRoute);
    }

    render() {
        return (
            <div className={`clusters ${this.state.moreInfoClicked ? "exit-right" : ""}`}>
                <Query query={GET_CLUSTERS_QUERY} pollInterval={3*60*1000} fetchPolicy="network-only">
                    {({loading, error, data}) => {
                        if (loading) {
                            return (<LoadingIcon className="loading-clusters" text={"Clusters"} icon={<Isvg
                                src={require('../../assets/svgs/review-cluster.svg')}/>}/>);
                        }
                        const errorMessage = error ? error.message : null;
                        const clusters = !error && data.clusters ? data.clusters : [];
                        const filteredItems = this.state.filter ? clusters.filter(item => defaultFilterWithKeys(item, this.state.filter, ["name"])) : clusters;

                        return (
                            <div>
                                {!errorMessage && clusters.length > 0 && <div className="widgets-cards-container">
                                    <div className="widgets-container">
                                        <ItemCountWidget className="float-left"
                                                         singularTitle={'Cluster'} pluralTitle={'Clusters'}
                                                         itemCount={clusters.length}
                                                         filteredItemCount={filteredItems.length}></ItemCountWidget>
                                        <SearchWidget className="float-right" placeholder="You can search by name"
                                                      setFilter={this.setFilter.bind(this)}></SearchWidget>
                                    </div>
                                    <div className="card-container">{filteredItems.map(cluster => (
                                        <ClusterCard cluster={cluster} key={cluster.clusterId}
                                                     moreInfoClicked={this.moreInfoClicked.bind(this)}
                                                     handleTierCluster={this.tierCluster.bind(this, cluster)}/>
                                    ))}
                                    </div>
                                </div>}
                                {!errorMessage && clusters.length === 0 && <div className="empty-dashboard">
                                    <div className="icon">
                                        <Isvg src={require('../../assets/svgs/review-cluster.svg')}/>
                                    </div>
                                    <button type="button" className="secondary"
                                            onClick={() => {routesObject.routes.push('/discover')}}>Discover Your First ONTAP Cluster
                                    </button>
                                </div>}
                                {errorMessage && <div className="error">
                                    <Isvg className="error-icon" cacheGetRequests={true} src={require("./../../assets/svgs/error.svg")}/>
                                    <span className="error-text">{errorMessage}</span>
                                </div>}
                            </div>
                        );
                    }}
                </Query>
            </div>
        )
    }
}

const mapStateToProps = state => {
    return {
        wizard: state.wizard,
        routing: state.router
    };
};


export default connect(mapStateToProps, routingInfoDispatcher)(ClustersDashboard);

