import React from 'react';
import ReactDOM from 'react-dom';
import {Provider} from 'react-redux'
import {ConnectedRouter, push} from "connected-react-router";
import {bindActionCreators} from "redux";
import {Route, Switch} from 'react-router-dom'
import {ApolloProvider} from 'react-apollo';
import {ApolloClient} from 'apollo-client';
import {createHttpLink} from 'apollo-link-http';
import {setContext} from 'apollo-link-context';
import {onError} from 'apollo-link-error'
import {defaultDataIdFromObject, InMemoryCache} from 'apollo-cache-inmemory'
// import {ReduxCache} from 'apollo-cache-redux';
import * as serviceWorker from './serviceWorker';
import App from './App';
import {history, store} from './store'
import * as actions from './actions';
import numeral from 'numeral';
import localStorage from 'local-storage'
import _ from 'lodash'
import './index.scss';

//handle auth issues
const auth0 = require('auth0-js');
const Auth = require('./auth/Auth.js').default;
window.auth0 = auth0;
const auth0Options = {
    clientID: process.env.REACT_APP_AUTH_CLIENT,
    domain: process.env.REACT_APP_AUTH_DOMAIN,
    audience: process.env.REACT_APP_AUTH_AUDIENCE
};
console.log(`%c connecting to auth0 with ${JSON.stringify(auth0Options)}`, 'font-weight:bold;color:green');
window.auth = new Auth(auth0Options);


actions.routesObject.routes = {push: bindActionCreators(push, store.dispatch)};
store.dispatch({type: actions.AUTH_REFRESH_SSO}); //start login flow

Object.assign(numeral.locales["en"].abbreviations, {thousand: "K", million: "M", billion: "B", trillion: "T"});

//initialize GraphQL
const CLOUD_TIERING_BASE_URL = `${document.location.protocol}//${document.location.hostname}/graphql`;
console.log(`%c baseUrl ${CLOUD_TIERING_BASE_URL}`, 'font-weight:bold; color:purple');

const cache = new InMemoryCache({
    dataIdFromObject: object => {
        switch (object.__typename) {
            case 'Cluster':
                return `${object.__typename}:${object.clusterId}`;
            case 'Volume':
                return `${object.__typename}:${object.name}:${object.svmName}`;
            case 'Region':
                return `${object.__typename}:${object.region}`;
            default:
                return defaultDataIdFromObject(object); // fall back to default handling
        }
    },
    cacheRedirects: {
        Query: {
            clusters: (_, args, {getCacheKey}) => getCacheKey({__typename: 'Cluster', clusterId: args.clusterId}),
            cluster: (_, args, {getCacheKey}) => getCacheKey({__typename: 'Cluster', clusterId: args.clusterId})
        }
    },
});

const httpLink = createHttpLink({
    uri: CLOUD_TIERING_BASE_URL,
});

const authLink = setContext((_, {headers}) => {
    // get the authentication token from local storage if it exists
    const token = localStorage.get('auth0LoginToken');
    // return the headers to the context so httpLink can read them
    return {
        headers: {
            ...headers,
            authorization: token || "",
        }
    }
});

const errorHandling = onError(({graphQLErrors, networkError}) => {
    if (networkError) {
        switch (networkError.statusCode) {
            case 500: {
                console.log("Internal error detected from server..\n\n" + networkError);
                break;
            }
            case 400: {
                if (_.get(networkError, "result.errors.extensions.code", "") === "UNAUTHENTICATED") {
                    console.log(`%c Failed authenticating user, error: ${_.get(networkError, "result, errors, extensions, exception, stacktrace", "")} logging out`, "font-weight: bold; color: red");
                }
                break;
            }
            case 401: {
                console.log(`%c Failed authenticating user, error: ${_.get(networkError, "result, errors, extensions, exception, stacktrace", "")} logging out`, "font-weight: bold; color: red");
                window.auth.logout();
                break;
            }
            default: {
            }
        }
    }
    if (graphQLErrors) {
        _.each(graphQLErrors, ({message, locations, extensions, path}) => {
            if (extensions.code === 401) {
                console.log(`%c Failed authenticating user, error: ${message} logging out`, "font-weight: bold; color: red");
                window.auth.logout();
            } else {
                console.log(`%c got GraphQL error ${message} in locations ${JSON.stringify(locations)} and path ${path} url ${JSON.stringify(window.location)}`, "font-weight: bold; color: red")
            }
        });
    }
});


const client = new ApolloClient({
    link: errorHandling.concat(authLink).concat(httpLink),
    cache,
    connectToDevTools: true
});

ReactDOM.render(
    <ApolloProvider client={client}>
        <Provider store={store}>
            <ConnectedRouter history={history}>
                <Switch>
                    <Route path="/" component={App}/>
                </Switch>
            </ConnectedRouter>
        </Provider>
    </ApolloProvider>,
    document.getElementById('root'));

// If you want your app to work offline and load faster, you can change
// unregister() to register() below. Note this comes with some pitfalls.
// Learn more about service workers: http://bit.ly/CRA-PWA
serviceWorker.unregister();
