import _ from 'lodash'
import {CLUSTER_STATUS, DASHBOARD_TITLES} from "./consts";
import numeral from "numeral";
import bytes from 'bytes'

export const defaultFilter = (item, filterString) => {
    if (_.isFunction(item)) return false;
    return _.values(item).find(value => {
        if (_.isEmpty(value) || !value) {
            return false;
        }
        else if(_.isObject(value)){
            return defaultFilter(value, filterString);
        }
        else if(_.isArray(value)){
            return false;
        }
        else return _.toString(value).toLowerCase().indexOf(filterString.toLowerCase()) !== -1;
    } );
};

export const defaultFilterWithKeys = (item, filterString, keys) => {
    if (_.isFunction(item)) return false;
    return _.keys(item).find(key => {
        if (!_.includes(keys, key)) return false; //not one of the keys to check
        const value = item[key];
        if (_.isEmpty(value) || !value) {
            return false;
        }
        else if(_.isObject(value)){
            return defaultFilterWithKeys(value, filterString, keys);
        }
        else if(_.isArray(value)){
            return false;
        }
        else return _.toString(value).toLowerCase().indexOf(filterString.toLowerCase()) !== -1;
    } );
};

export const calculateNextStep = (wizard) => {
    const currentStepIndex = wizard.stepNames.indexOf(wizard.currentStep);
    const nextStepName = wizard.stepNames[currentStepIndex+1];
    // console.log(`%c calculateNextStep currentStep=${wizard.currentStep}, curStepIndex=${currentStepIndex} nextStepName=${nextStepName}`, "font-weight: bold; color: purple");
    return wizard.stepNameToPathName[nextStepName];
};

export const calculateSavings = (cluster) => {
    const {status, tierInformation, capacityInformation} = cluster;
    const clusterIsOff = status && status.status !== CLUSTER_STATUS.ON;

    const currentSavingsCapacity = buildCapacityObject(_.get(tierInformation, "tieredCapacity", 0));

    const currentSavings = [];
    if (clusterIsOff) currentSavings.push({italicText: DASHBOARD_TITLES.dataNotAvailable});
    else if (currentSavingsCapacity.bytes > 0) {
        currentSavings.push({...currentSavingsCapacity});
        currentSavings.push({value: `$ ${numeral(_.get(tierInformation, "savings", 0)).format('0,0')}`});
    }
    else currentSavings.push({italicText: DASHBOARD_TITLES.notTiered});
    const savingOpportunityCapacity = buildCapacityObject(_.get(capacityInformation, "performanceCapacity.inactiveCapacity", 0));
    const additionalSavings = [];
    if (clusterIsOff) additionalSavings.push({italicText: DASHBOARD_TITLES.dataNotAvailable});
    else {
        const {volumes: {eligibleVolumes, tieredVolumes}} = cluster;
        if (savingOpportunityCapacity.bytes > 0) additionalSavings.push({...savingOpportunityCapacity});
        else if (tieredVolumes && tieredVolumes === 0) additionalSavings.push({italicText: DASHBOARD_TITLES.notTiered});
        else if (tieredVolumes && eligibleVolumes && tieredVolumes === eligibleVolumes) additionalSavings.push({italicText : DASHBOARD_TITLES.allTiered});
        else additionalSavings.push({italicText: DASHBOARD_TITLES.dataNotAvailable});
    }
    return {currentSavings, additionalSavings};
};

export const buildCapacityObject = (bytesInput) => {
    if (bytesInput === null) return bytesInput;
    const converted = bytes(bytesInput, {unitSeparator: " "});
    const convertedArray = converted ? converted.split(" ") : ["0", "B"];
    // console.log(`%c converted1 ${converted} convertedArray ${convertedArray}`, 'font-weight:bold; color:blue');
    return {
        bytes: bytesInput,
        value: convertedArray[0],
        unit: convertedArray[1]
    };
};

export const extractONTAPVersion = (versionString) => {
    const regex = /(\d+\.\d*|\d+)/;
    const version = versionString.match(regex)[0];
    return parseFloat(version);
};



